/*
 *  Copyright 2020-2023 Google LLC
 *  Copyright 2020-2023 EPAM Systems, Inc
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.opengroup.osdu.notification.pubsub;

import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.oqm.core.OqmDriver;
import org.opengroup.osdu.oqm.core.model.*;
import org.opengroup.osdu.notification.service.ExternalSubscriptionsManager;
import org.opengroup.osdu.notification.pubsub.receiver.OqmEventReplicator;

import java.util.Collections;
import java.util.HashMap;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class OqmEventReplicatorTest {

    public static final String TEST_TENANT = "tenant1";
    public static final String TEST_TOPIC = "topic1";
    @InjectMocks
    private OqmEventReplicator sut;
    @Mock
    private OqmAckReplier replier;
    @Mock
    private OqmDriver driver;
    @Mock
    private OqmSubscription oqmSubscription;
    @Mock
    private ExternalSubscriptionsManager externalSubscriptionsManager;
    private Set<Subscription> subscriptionInfos;
    private OqmTopic topic;

    @Before
    public void setUp() {
        topic = OqmTopic.builder().name(TEST_TOPIC).build();
        Subscription subscription1 = new Subscription();
        subscription1.setTopic(topic.getName());
        subscription1.setId("1");
        Subscription subscription2 = new Subscription();
        subscription2.setTopic(topic.getName());
        subscription2.setId("2");
        Subscription subscription3 = new Subscription();
        subscription3.setTopic(topic.getName());
        subscription3.setId("3");
        subscriptionInfos = Stream.of(subscription1, subscription2, subscription3).collect(Collectors.toSet());
    }

    @Test
    public void testReceiveMessageSubscriptionCreated() {
        when(oqmSubscription.getTopics()).thenReturn(Collections.singletonList(topic));
        when(externalSubscriptionsManager.getSubscriptionsForTopic(TEST_TENANT, TEST_TOPIC)).thenReturn(subscriptionInfos);
        OqmMessage message = OqmMessage.builder().data("Message").attributes(new HashMap<String, String>() {{
            put("data-partition-id", TEST_TENANT);
        }}).build();
        sut.receiveMessage(message, replier);
        verify(driver, times(3)).publish(any(OqmMessage.class), any(), any());
        verify(replier, times(1)).ack();
    }
}