// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.provider.azure.messageBus;

import com.microsoft.azure.servicebus.IMessage;
import com.microsoft.azure.servicebus.SubscriptionClient;
import com.microsoft.azure.servicebus.management.SubscriptionDescription;
import org.opengroup.osdu.azure.servicebus.AbstractMessageHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MessageHandler extends AbstractMessageHandler {

    private final static Logger LOGGER = LoggerFactory.getLogger(MessageHandler.class);
    private ProcessNotification processNotification;
    private String subscriptionName;
    private SubscriptionClient subscriptionClient;
    private int maxDeliveryCount;
    private boolean deleteMessageAfterMaxDeliveryCount;


    public MessageHandler(SubscriptionClient client, ProcessNotification processNotification, String appName, SubscriptionDescription subscriptionDescription, boolean deleteMessageAfterMaxDeliveryCount) {
        super(appName, client);
        this.subscriptionName = client.getSubscriptionName();
        this.processNotification = processNotification;
        this.subscriptionClient = client;
        this.maxDeliveryCount = subscriptionDescription.getMaxDeliveryCount();
        this.deleteMessageAfterMaxDeliveryCount = deleteMessageAfterMaxDeliveryCount;
    }

    public void processMessage(IMessage message) throws Exception {
        try {
            this.processNotification.performNotification(message, subscriptionName, subscriptionClient.getTopicName());
        } catch (Exception e) {
            int messageDeliveryCount = (int) message.getDeliveryCount();
            if (messageDeliveryCount >= maxDeliveryCount && deleteMessageAfterMaxDeliveryCount) {
                LOGGER.error(String.format("Max Delivery Count of %d Exceeded for Message with id : %s for Subscription : %s; Completing the message.", maxDeliveryCount, message.getMessageId(), subscriptionName), e);
                subscriptionClient.complete(message.getLockToken());
            } else {
                LOGGER.error(String.format("Attempt %d/%d failed.", message.getDeliveryCount(), maxDeliveryCount), e);
                throw e;
            }
        }
    }
}
