// Copyright 2017-2025, SLB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.provider.azure.util;

import com.google.common.base.Strings;
import com.microsoft.applicationinsights.TelemetryClient;
import com.microsoft.applicationinsights.telemetry.MetricTelemetry;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.notification.provider.interfaces.IMetricService;
import org.opengroup.osdu.notification.utils.Config;
import org.springframework.stereotype.Component;

@Component
public class MetricServiceImpl implements IMetricService {
    private static final String LATENCY_METRIC_NAME = "[Notification] Event notification latency";
    private final TelemetryClient telemetryClient = new TelemetryClient();

    @Override
    public void sendLatencyMetric(long latency, String dataPartitionId, String correlationId, String topicName) {
        this.sendMetric(LATENCY_METRIC_NAME, latency, dataPartitionId, correlationId, topicName);
    }

    private void sendMetric(String name, long value, String dataPartitionId, String correlationId, String topicName) {
        MetricTelemetry metric = new MetricTelemetry();
        metric.setName(name);
        metric.setValue(value);
        if (!Strings.isNullOrEmpty(dataPartitionId)) {
            metric.getProperties().putIfAbsent(DpsHeaders.DATA_PARTITION_ID, dataPartitionId);
        }
        if (!Strings.isNullOrEmpty(topicName)) {
            metric.getProperties().putIfAbsent(Config.TOPIC_NAME, topicName);
        }
        metric.getProperties().putIfAbsent(DpsHeaders.CORRELATION_ID, correlationId);
        telemetryClient.trackMetric(metric);
        telemetryClient.flush();
    }
}
